import { AbstractDynamicSnippetsGeneratorContext, FernGeneratorExec } from '@fern-api/browser-compatible-base-generator';
import { go, BaseGoCustomConfigSchema } from '@fern-api/go-ast';
import { dynamic, TypeId, Name, FernFilepath } from '@fern-fern/ir-sdk/api';
import { TypeInstance } from './TypeInstance.js';
import { DiscriminatedUnionTypeInstance } from './DiscriminatedUnionTypeInstance.js';
import { ErrorReporter } from './context/ErrorReporter.js';

declare namespace DynamicTypeMapper {
    interface Args {
        typeReference: dynamic.TypeReference;
    }
}
declare class DynamicTypeMapper {
    private context;
    constructor({ context }: {
        context: DynamicSnippetsGeneratorContext;
    });
    convert(args: DynamicTypeMapper.Args): go.Type;
    private convertLiteral;
    private convertNamed;
    private convertUnknown;
    private convertPrimitive;
}

declare namespace DynamicTypeInstantiationMapper {
    interface Args {
        typeReference: dynamic.TypeReference;
        value: unknown;
        as?: ConvertedAs;
    }
    type ConvertedAs = "key";
}
declare class DynamicTypeInstantiationMapper {
    private context;
    constructor({ context }: {
        context: DynamicSnippetsGeneratorContext;
    });
    convert(args: DynamicTypeInstantiationMapper.Args): go.TypeInstantiation;
    private convertList;
    private convertMap;
    private convertNamed;
    private convertDiscriminatedUnion;
    private getBaseFields;
    private convertObject;
    private convertEnum;
    private getEnumValueName;
    private convertUndicriminatedUnion;
    private findMatchingUndiscriminatedUnionType;
    private getUndiscriminatedUnionFieldName;
    private getUndiscriminatedUnionFieldNameForList;
    private getUndiscriminatedUnionFieldNameForMap;
    private getUndiscriminatedUnionFieldNameForOptional;
    private getUndiscriminatedUnionFieldNameForSet;
    private getUndiscriminatedUnionFieldNameForLiteral;
    private getUndiscriminatedUnionFieldNameForPrimitive;
    private convertUnknown;
    private convertPrimitive;
    private getValueAsNumber;
    private getValueAsBoolean;
    private getValueAsString;
    private newTypeMismatchError;
}

interface FilePropertyInfo {
    fileFields: go.StructField[];
    bodyPropertyFields: go.StructField[];
}
declare class FilePropertyMapper {
    private context;
    constructor({ context }: {
        context: DynamicSnippetsGeneratorContext;
    });
    getFilePropertyInfo({ body, value }: {
        body: dynamic.FileUploadRequestBody;
        value: unknown;
    }): FilePropertyInfo;
    private getSingleFileProperty;
    private getArrayFileProperty;
    private getBodyProperty;
}

declare class DynamicSnippetsGeneratorContext extends AbstractDynamicSnippetsGeneratorContext<dynamic.DynamicIntermediateRepresentation> {
    customConfig: BaseGoCustomConfigSchema | undefined;
    errors: ErrorReporter;
    dynamicTypeMapper: DynamicTypeMapper;
    dynamicTypeInstantiationMapper: DynamicTypeInstantiationMapper;
    filePropertyMapper: FilePropertyMapper;
    rootImportPath: string;
    private httpEndpointReferenceParser;
    constructor({ ir, config }: {
        ir: dynamic.DynamicIntermediateRepresentation;
        config: FernGeneratorExec.GeneratorConfig;
    });
    clone(): DynamicSnippetsGeneratorContext;
    associateQueryParametersByWireValue({ parameters, values }: {
        parameters: dynamic.NamedParameter[];
        values: dynamic.Values;
    }): TypeInstance[];
    associateByWireValue({ parameters, values, ignoreMissingParameters }: {
        parameters: dynamic.NamedParameter[];
        values: dynamic.Values;
        ignoreMissingParameters?: boolean;
    }): TypeInstance[];
    isFileUploadRequestBody(body: dynamic.InlinedRequestBody): body is dynamic.InlinedRequestBody.FileUpload;
    needsRequestParameter({ request }: {
        request: dynamic.InlinedRequest;
    }): boolean;
    includePathParametersInWrappedRequest({ request }: {
        request: dynamic.InlinedRequest;
    }): boolean;
    private includeRequestBodyInWrappedRequest;
    private includeFileUploadBodyInWrappedRequest;
    private fileUploadHasBodyProperties;
    private fileUploadHasFileProperties;
    getRecord(value: unknown): Record<string, unknown> | undefined;
    resolveNamedType({ typeId }: {
        typeId: TypeId;
    }): dynamic.NamedType | undefined;
    resolveDiscriminatedUnionTypeInstance({ discriminatedUnion, value }: {
        discriminatedUnion: dynamic.DiscriminatedUnionType;
        value: unknown;
    }): DiscriminatedUnionTypeInstance | undefined;
    getMethodName(name: Name): string;
    getTypeName(name: Name): string;
    getImportPath(fernFilepath: FernFilepath): string;
    getContextTypeReference(): go.TypeReference;
    getContextTodoFunctionInvocation(): go.FuncInvocation;
    getIoReaderTypeReference(): go.TypeReference;
    getNewStringsReaderFunctionInvocation(s: string): go.FuncInvocation;
    getClientConstructorName(): string;
    getClientImportPath(): string;
    getOptionImportPath(): string;
    resolveEndpointOrThrow(rawEndpoint: string): dynamic.Endpoint[];
    resolveEndpointLocationOrThrow(location: dynamic.EndpointLocation): dynamic.Endpoint[];
    getGoTypeReferenceFromDeclaration({ declaration }: {
        declaration: dynamic.Declaration;
    }): go.TypeReference;
    newParameterNotRecognizedError(parameterName: string): Error;
    private isListTypeReference;
    private parsedEndpointMatches;
}

export { DynamicSnippetsGeneratorContext as D, type FilePropertyInfo as F, DynamicTypeMapper as a, DynamicTypeInstantiationMapper as b, FilePropertyMapper as c };
